package com.ekexiu.project.bridge.system.service;

import com.ekexiu.project.bridge.resource.po.Bridge;
import com.ekexiu.project.bridge.resource.po.BridgeServer;
import com.ekexiu.project.bridge.resource.po.CollectDevice;
import com.ekexiu.project.bridge.resource.po.Transducer;
import com.ekexiu.project.bridge.system.dao.BridgeDao;
import com.ekexiu.project.bridge.system.dao.BridgeServerDao;
import com.ekexiu.project.bridge.system.dao.CollectDeviceDao;
import com.ekexiu.project.bridge.system.dao.TransducerDao;
import com.ekexiu.project.bridge.system.dao.UserDao;
import com.ekexiu.project.bridge.system.po.User;
import com.ekexiu.project.bridge.system.vo.SessionUser;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.method.InvalidSession;
import org.jfw.apt.web.annotation.method.SetSession;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.SessionVal;
import org.jfw.apt.web.annotation.param.Upload;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.web.fileupload.Item;
import org.jfw.util.web.fileupload.UploadItemIterator;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Created by TT on 2018/8/7.
 */
@Path
public class SysService {

    public static final String DEFAULT_PW_STR = StringUtil.md5("123456");

    private static final AtomicInteger FN_IDX = new AtomicInteger(1);

    private File imgPath;

    @Autowrie
    private UserDao userDao;
    @Autowrie
    private BridgeDao bridgeDao;
    @Autowrie
    private BridgeServerDao bridgeServerDao;
    @Autowrie
    private CollectDeviceDao collectDeviceDao;
    @Autowrie
    private TransducerDao transducerDao;

    public BridgeDao getBridgeDao() {
        return bridgeDao;
    }

    public void setBridgeDao(BridgeDao bridgeDao) {
        this.bridgeDao = bridgeDao;
    }

    public BridgeServerDao getBridgeServerDao() {
        return bridgeServerDao;
    }

    public void setBridgeServerDao(BridgeServerDao bridgeServerDao) {
        this.bridgeServerDao = bridgeServerDao;
    }

    public CollectDeviceDao getCollectDeviceDao() {
        return collectDeviceDao;
    }

    public void setCollectDeviceDao(CollectDeviceDao collectDeviceDao) {
        this.collectDeviceDao = collectDeviceDao;
    }

    public TransducerDao getTransducerDao() {
        return transducerDao;
    }

    public void setTransducerDao(TransducerDao transducerDao) {
        this.transducerDao = transducerDao;
    }

    public UserDao getUserDao() {
        return userDao;
    }

    public void setUserDao(UserDao userDao) {
        this.userDao = userDao;
    }

    /**
     * 上传文件
     *
     * @param it
     * @return
     * @throws Exception
     */
    @Path("/bridge/upload")
    @Post
    public List<UploadFile> upload(@Upload UploadItemIterator it) throws Exception {
        List<UploadFile> ret = new LinkedList<UploadFile>();
        try {
            while (it.hasNext()) {
                Item item = it.next();
                if (!item.isFormField()) {
                    String name = normalizeFileName(item.getName());
                    int index = name.lastIndexOf('.');
                    String ext = index >= 0 ? name.substring(index + 1) : "";
                    ext = ext.trim();
                    long fileLen = 0;
                    int len = 0;
                    UploadFile uf = this.buildTargetFile(ext);
                    uf.setName(name);
                    File file = uf.getFn();
                    ret.add(uf);
                    byte[] buf = new byte[8092];
                    OutputStream of = new FileOutputStream(file);
                    try {
                        InputStream in = item.getInputStream();
                        try {
                            while ((len = in.read(buf)) >= 0) {
                                if (len > 0) {
                                    of.write(buf, 0, len);
                                    fileLen += len;
                                }
                            }
                            uf.setSize(fileLen);
                        } finally {
                            in.close();
                        }
                    } finally {
                        of.close();
                    }
                }
            }
            return ret;
        } catch (Exception e) {
            this.remove(ret);
            throw e;
        } finally {
            if (it != null)
                it.clean();
        }
    }


    @SetSession("JFW_SESSION_LOGIN_USER=result")
    @Path("/user/login")
    @Post
    public SessionUser login(@JdbcConn Connection con, String account, String pw,String vc,@Nullable @SessionVal("PIC_LOGIN") String code,@DefaultValue("0") @SessionVal("TIMEOUT_PIC_LOGIN") long timeout) throws SQLException {
       if(code ==null){
    	   return null;
       }
       if(!vc.equals(code)){
    	   return null;
       }
       if(System.currentTimeMillis()>timeout){
    	   return null;
       }
    	User user = userDao.login(con, account, StringUtil.md5(pw));
        if (user != null) {
            return makeSessionUser(user);
        }
        return null;
    }

    @Get
    @Path("/user")
    public SessionUser get(@LoginUser SessionUser user) {
        return user;
    }

    @Get
    @Path("/user/logout")
    @InvalidSession
    public void logout() {
    }

    @Post
    @Path("/user/insert")
    public void insert(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, User user) throws SQLException {
        user.setId(StringUtil.buildUUID());
        user.setActive(true);
        user.setPasswd(DEFAULT_PW_STR);
        user.setType(0);
        user.setCreator(sessionUser.getId());
        user.setModifier(sessionUser.getId());
        this.userDao.insert(con, user);
    }

    @Get
    @Path("/user/qo")
    public SessionUser query(@JdbcConn Connection con, String id) throws SQLException {
        return makeSessionUser(this.userDao.query(con, id));
    }

    @Post
    @Path("/user/update")
    public void update(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, User user) throws SQLException {
        user.setModifier(sessionUser.getId());
        this.userDao.update(con, user);
    }

    @Get
    @Path("/user/ban")
    public void ban(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, String id) throws SQLException {
        this.userDao.ban(con, sessionUser.getId(), id);
    }

    @Get
    @Path("/user/pq")
    public PageQueryResult<SessionUser> pageQuery(@JdbcConn Connection con, @Nullable String account, @Nullable String name, @Nullable String comp, int pageSize, int pageNo) throws SQLException {
        PageQueryResult<User> pageQueryResult = this.userDao.pageQuery(con, account, name == null ? null : "%" + name + "%", comp == null ? null : "%" + comp + "%", pageSize, pageNo);
        List<User> users = pageQueryResult.getData();
        List<SessionUser> sessionUsers = new ArrayList<>();
        if (!users.isEmpty()) {
            for (User user : users) {
                sessionUsers.add(makeSessionUser(user));
            }
            PageQueryResult<SessionUser> ret = new PageQueryResult<>();
            ret.setPageNo(pageQueryResult.getPageNo());
            ret.setPageSize(pageQueryResult.getPageSize());
            ret.setTotal(pageQueryResult.getTotal());
            
            ret.setData(sessionUsers);
            return ret;
        }
        return null;
    }

    @Get
    @Path("/user/resetPw")
    public void resetPw(@JdbcConn(true) Connection con, String id) throws SQLException {
        this.userDao.changePw(con, DEFAULT_PW_STR, id);
    }

    @Post
    @Path("/user/changePw")
    public void changePw(@JdbcConn(true) Connection con, String id, String oldPw, String newPw) throws SQLException, JfwBaseException {
        User user = this.userDao.query(con, id);
        if (user != null && StringUtil.md5(oldPw).equals(StringUtil.md5(user.getPasswd()))) {
            this.userDao.changePw(con, StringUtil.md5(newPw), id);
        } else {
            throw new JfwBaseException(100001, "用户不存在或密码不匹配");
        }
    }

    @Post
    @Path("/bridge")
    public String insert(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, Bridge bridge) throws SQLException {
        String id = StringUtil.buildUUID();
        bridge.setId(id);
        bridge.setActive(true);
        bridge.setCreator(sessionUser.getId());
        bridge.setModifier(sessionUser.getId());
        this.bridgeDao.insert(con, bridge);
        return id;
    }

    @Get
    @Path("/bridge/byUser")
    public PageQueryResult<Bridge> query(@JdbcConn Connection con,String uid,@Nullable Boolean active,int pageSize,int pageNo)throws SQLException{
    	return this.bridgeDao.pageQuery(con, active, uid, pageSize, pageNo);
    }
    
    @Get
    @Path("/bridge/qo")
    public Bridge queryBridge(@JdbcConn Connection con, String id) throws SQLException {
        return this.bridgeDao.query(con, id);
    }

    @Get
    @Path("/bridge/delete")
    public void delete(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, String id) throws SQLException {
        this.bridgeDao.logicDelete(con, sessionUser.getId(), id);
    }

    @Post
    @Path("/bridge/update")
    public void update(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, Bridge bridge) throws SQLException {
        bridge.setModifier(sessionUser.getId());
        this.bridgeDao.update(con, bridge);
    }

    @Get
    @Path("/bridge/pq")
    public PageQueryResult<Bridge> pageQuery(@JdbcConn Connection con, @Nullable String name, @Nullable String code, int pageSize, int pageNo) throws SQLException {
        return this.bridgeDao.pageQuery(con, name == null ? null : "%" + name + "%", code, pageSize, pageNo);
    }

    @Post
    @Path("/bridgeServer")
    public String insert(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, BridgeServer bridgeServer) throws SQLException {
        String id = StringUtil.buildUUID();
        bridgeServer.setId(id);
        bridgeServer.setActive(true);
        bridgeServer.setCreator(sessionUser.getId());
        bridgeServer.setModifier(sessionUser.getId());
        this.bridgeServerDao.insert(con, bridgeServer);
        return id;
    }

    @Get
    @Path("/bridgeServer/qo")
    public BridgeServer queryBS(@JdbcConn Connection con, String id) throws SQLException {
        return this.bridgeServerDao.query(con, id);
    }

    @Get
    @Path("/bridgeServer/pq")
    public PageQueryResult<BridgeServer> pageQueryBS(@JdbcConn Connection con, @Nullable String bcode, @Nullable String code, int pageSize, int pageNo) throws SQLException {
        return this.bridgeServerDao.pageQuery(con, bcode, code, pageSize, pageNo);
    }

    @Post
    @Path("/bridgeServer/update")
    public void updateBS(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, BridgeServer bridgeServer) throws SQLException {
        bridgeServer.setModifier(sessionUser.getId());
        this.bridgeServerDao.update(con, bridgeServer);
    }

    @Get
    @Path("/bridgeServer/delete")
    public void deleteBS(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser
            , String id) throws SQLException {
        this.bridgeServerDao.logicDelete(con, sessionUser.getId(), id);
    }

    @Post
    @Path("/collectDevice")
    public String insert(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, CollectDevice collectDevice) throws SQLException {
        String id = StringUtil.buildUUID();
        collectDevice.setId(id);
        collectDevice.setActive(true);
        collectDevice.setCreator(sessionUser.getId());
        collectDevice.setModifier(sessionUser.getId());
        this.collectDeviceDao.insert(con, collectDevice);
        return id;
    }

    @Get
    @Path("/collectDevice/qo")
    public CollectDevice queryCD(@JdbcConn Connection con, String id) throws SQLException {
        return this.collectDeviceDao.query(con, id);
    }

    @Get
    @Path("/collectDevice/pq")
    public PageQueryResult<CollectDevice> pageQueryCD(@JdbcConn Connection con, @Nullable String scode, @Nullable String code, int pageSize, int pageNo) throws SQLException {
        return this.collectDeviceDao.pageQuery(con, scode, code, pageSize, pageNo);
    }

    @Post
    @Path("/collectDevice/update")
    public void updateCD(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, CollectDevice collectDevice) throws SQLException {
        collectDevice.setModifier(sessionUser.getId());
        this.collectDeviceDao.update(con, collectDevice);
    }

    @Get
    @Path("/collectDevice/delete")
    public void deleteCD(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, String id) throws SQLException {
        this.collectDeviceDao.logicDelete(con, sessionUser.getId(), id);
    }

    @Post
    @Path("/transducer")
    public String insert(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, Transducer transducer) throws SQLException {
        String id = StringUtil.buildUUID();
        transducer.setId(id);
        transducer.setActive(true);
        transducer.setCreator(sessionUser.getId());
        transducer.setModifier(sessionUser.getId());
        this.transducerDao.insert(con, transducer);
        return id;
    }

    @Get
    @Path("/transducer/qo")
    public Transducer queryTransducer(@JdbcConn Connection con, String id) throws SQLException {
        return this.transducerDao.query(con, id);
    }

    @Get
    @Path("/transducer/pq")
    public PageQueryResult<Transducer> pageQueryT(@JdbcConn Connection con, @Nullable String cdCode, @Nullable String code, int pageSize, int pageNo) throws SQLException {
        return this.transducerDao.pageQuery(con, cdCode, code, pageSize, pageNo);
    }

    @Post
    @Path("/transducer/update")
    public void updateTransducer(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, Transducer transducer) throws SQLException {
        transducer.setModifier(sessionUser.getId());
        this.transducerDao.update(con, transducer);
    }

    @Get
    @Path("/transducer/delete")
    public void deleteTransducer(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, String id) throws SQLException {
        this.transducerDao.logicDelete(con, sessionUser.getId(), id);
    }

    @Post
    @Path("/notice")
    public void updateNotice(@JdbcConn(true)Connection con,@LoginUser SessionUser sessionUser,String cnt)throws SQLException {
        this.userDao.updateNotice(con, cnt, sessionUser.getId());
    }


    private void remove(List<UploadFile> list) {
        for (UploadFile file : list) {
            if (file.fn != null)
                file.fn.delete();
        }
    }

    private UploadFile buildTargetFile(String ext) {
        StringBuilder sb = new StringBuilder();
        sb.append(System.currentTimeMillis() / (1000 * 60 * 60 * 24));
        File path = new File(this.imgPath, sb.toString());
        if (!path.exists()) {
            synchronized (this) {
                if (!path.mkdirs())
                    throw new RuntimeException("mkdir error[" + path + "]");
                FN_IDX.set(1);
            }
        }
        File file;
        do {
            String fn = FN_IDX.toString();
            if (ext.isEmpty()) {
                file = new File(path, fn);
            } else {
                file = new File(path, fn + "." + ext);
            }
            FN_IDX.incrementAndGet();
        } while (file.exists());
        sb.append("/").append(file.getName());
        UploadFile uf = new UploadFile();
        uf.setFn(file);
        uf.setUri("/" + sb.toString());
        return uf;
    }

    private String normalizeFileName(String fn) {
        int index = fn.indexOf('\\');
        if (index >= 0) {
            fn = fn.substring(fn.lastIndexOf('\\') + 1);
        }
        index = fn.indexOf('/');
        if (index >= 0) {
            fn = fn.substring(fn.lastIndexOf('/') + 1);
        }
        if (fn.length() == 0)
            throw new RuntimeException("invalid filename in Multipart/data request");
        return fn;
    }

    public static class UploadFile {
        private String name;
        private String uri;
        private long size;
        private transient File fn;

        public File getFn() {
            return fn;
        }

        public void setFn(File fn) {
            this.fn = fn;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public String getUri() {
            return uri;
        }

        public void setUri(String uri) {
            this.uri = uri;
        }

        public long getSize() {
            return size;
        }

        public void setSize(long size) {
            this.size = size;
        }
    }

    private static SessionUser makeSessionUser(User user) {
        SessionUser sessionUser = new SessionUser();
        sessionUser.setAccount(user.getAccount());
        sessionUser.setComp(user.getComp());
        sessionUser.setEmail(user.getEmail());
        sessionUser.setId(user.getId());
        sessionUser.setJob(user.getJob());
        sessionUser.setName(user.getName());
        sessionUser.setPhone(user.getPhone());
        sessionUser.setRemark(user.getRemark());
        sessionUser.setType(user.getType());
        return sessionUser;
    }
}
